import { FormEvent, useCallback, useRef, useState } from "react"
import "../style/settings.css"
import { useAppFetcher, useUser } from "../App.tsx"
import { Fetcher } from "../app/Fetcher.ts"

export default function ProfileSettings() {
    const fetcher = useAppFetcher()
    const [user, setUser] = useUser()

    const [errorMessages, setErrorMessages] = useState<string[]>([])
    const [success, setSuccess] = useState(false)

    const formRef = useRef<HTMLFormElement | null>(null)

    const submitForm = useCallback(
        async (e: FormEvent) => {
            e.preventDefault()
            const { name, email, password, confirmPassword } =
                Object.fromEntries<string>(
                    new FormData(formRef.current!) as Iterable<
                        [PropertyKey, string]
                    >,
                )

            if (password !== confirmPassword) {
                setErrorMessages(["Les mots de passe ne correspondent pas !"])
                return
            }

            const req: AccountUpdateRequest = {
                name: name,
                email: email,
            }
            if (password && password.length !== 0) {
                req.password = password
            }
            const errors = await updateAccount(fetcher, req)
            if (errors.length !== 0) {
                setErrorMessages(errors)
                setSuccess(false)
                return
            }

            setUser({ ...user!, email, name })
            setSuccess(true)
            formRef.current!.reset()
            setErrorMessages([])
        },
        [fetcher, setUser, user],
    )

    const [modalShow, setModalShow] = useState(false)
    const width = 150

    const profilePicture = user!.profilePicture
    return (
        <div id="settings-page">
            <div id="settings-content">
                {errorMessages.map((msg) => (
                    <div key={msg} className={"error-message"}>
                        {msg}
                    </div>
                ))}
                {success && (
                    <p className={"success-message"}>
                        Modifications sauvegardées
                    </p>
                )}
                <div id="settings-inputs">
                    <div id="settings-profile-picture">
                        <div>
                            <div id="profile-picture">
                                <img
                                    id="profile-picture-img"
                                    src={profilePicture}
                                    width={width}
                                    height={width}
                                    alt="profile-picture"
                                />
                            </div>
                            <button
                                className="settings-button"
                                onClick={() => setModalShow(true)}>
                                Changer photo de profil
                            </button>
                            <ProfileImageInputPopup
                                show={modalShow}
                                onHide={() => setModalShow(false)}
                            />
                        </div>
                    </div>
                    <div>
                        <form
                            ref={formRef}
                            id="credentials-form"
                            onSubmit={submitForm}>
                            <p>Nom d'utilisateur</p>
                            <input
                                className="settings-input"
                                name="name"
                                type="text"
                                placeholder={"Nom d'utilisateur"}
                                defaultValue={user!.name}
                            />

                            <p>Addresse email</p>
                            <input
                                className="settings-input"
                                name="email"
                                type="email"
                                placeholder={"Addresse email"}
                                defaultValue={user!.email}
                            />

                            <p>Mot de passe</p>
                            <input
                                className="settings-input"
                                name="password"
                                type="password"
                                placeholder={"Mot de passe"}
                            />

                            <p>Confirmez le mot de passe</p>
                            <input
                                className="settings-input"
                                name="confirmPassword"
                                type="password"
                                placeholder={"Confirmation du mot de passe"}
                            />

                            <button
                                className="settings-button"
                                type="submit"
                                onClick={submitForm}>
                                Mettre à jour
                            </button>
                        </form>
                    </div>
                </div>
            </div>
        </div>
    )
}

interface AccountUpdateRequest {
    name?: string
    email?: string
    profilePicture?: string
    password?: string
}

async function updateAccount(
    fetcher: Fetcher,
    req: AccountUpdateRequest,
): Promise<string[]> {
    const response = await fetcher.fetchAPI("user", req, "PUT")
    if (response.ok) return []
    const body = await response.json()
    return Object.entries(body).flatMap(([kind, messages]) =>
        (messages as string[]).map((msg) => `${kind}: ${msg}`),
    )
}

interface ProfileImageInputPopupProps {
    show: boolean
    onHide: () => void
}

function ProfileImageInputPopup({ show, onHide }: ProfileImageInputPopupProps) {
    const urlRef = useRef<HTMLInputElement>(null)
    const [errorMessages, setErrorMessages] = useState<string[]>()

    const fetcher = useAppFetcher()
    const [user, setUser] = useUser()

    if (!show) return <></>

    return (
        <div id="profile-picture-popup">
            <div id="profile-picture-popup-content">
                <div id="profile-picture-popup-header">
                    <p id="profile-picture-popup-title">
                        Nouvelle photo de profil
                    </p>
                </div>

                {errorMessages?.map((msg) => (
                    <div key={msg} className="error-message">
                        {msg}
                    </div>
                ))}
                <p id="profile-picture-popup-subtitle">
                    Saisissez le lien vers votre nouvelle photo de profil
                </p>
                <input
                    className={
                        `settings-input ` +
                        ((errorMessages?.length ?? 0) === 0
                            ? ""
                            : "invalid-input")
                    }
                    ref={urlRef}
                    type="input"
                    placeholder={"lien vers une image"}
                />
                <div id="profile-picture-popup-footer">
                    <button className={"settings-button"} onClick={onHide}>
                        Annuler
                    </button>
                    <button
                        className={"settings-button"}
                        onClick={async () => {
                            const url = urlRef.current!.value
                            const errors = await updateAccount(fetcher, {
                                profilePicture: url,
                            })
                            if (errors.length !== 0) {
                                setErrorMessages(errors)
                                return
                            }
                            setUser({ ...user!, profilePicture: url })
                            setErrorMessages([])
                            onHide()
                        }}>
                        Valider
                    </button>
                </div>
            </div>
        </div>
    )
}
